
/* --COPYRIGHT--,BSD
 * Copyright (c) 2014, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/


#include <msp430.h> 
#include <stdint.h>
#include "Read.h"


/*
 * 1) Set the SMCLK speed and desired baudrate.

   The baudrate is determined by the UCA1BR0, UCA1BR1, and UCA1MCTL registers.
   UCA1MCTL has three relevant fields:  UCA1BRF, UCA1BRS, and UCOS16.  The
   settings below are for:
   SMCLK = 8MHz
   baudrate = 28.8kbps

   If you change the SMCLK speed, or want a different baudrate, you need to
   change these values.  An easy way to determine them is the calculator at
   the MSP430 wiki page:
 *
 */
///BC UART Defines for baudrate: 28800
/*
#define UCA1_OS   1    // 1 = oversampling mode, 0 = low-freq mode
#define UCA1_BR0  17   // Value of UCA1BR0 register
#define UCA1_BR1  0    // Value of UCA1BR1 register
#define UCA1_BRS  0    // Value of UCBRS field in UCA1MCTL register
#define UCA1_BRF  6    // Value of UCBRF field in UCA1MCTL register
*/
#define bcUART_RX UCA1RXBUF


//   9600 Baud @ 2 MHz

#define UCA1_OS   0    // 1 = oversampling mode, 0 = low-freq mode
#define UCA1_BR0  208   // Value of UCA1BR0 register
#define UCA1_BR1  0    // Value of UCA1BR1 register
#define UCA1_BRS  3    // Value of UCBRS field in UCA1MCTL register
#define UCA1_BRF  0    // Value of UCBRF field in UCA1MCTL register


//LED for Debug
#define LED1Tog() 	P1OUT ^= BIT0;
#define LED2Tog()	P4OUT ^= BIT7;
#define LEDRecTog()	P1OUT ^= BIT6;
#define LED1On() 	P1OUT |= BIT0;
#define LED2On()	P4OUT |= BIT7;
#define LEDRecOn()	P1OUT |= BIT6;
#define LEDRecOff() P1OUT &= ~(BIT6);

uint8_t Stage =0;
uint8_t bcBuffCount=0; //Only bcUART ISR touch this

uint8_t chipErase =0;
uint8_t Sample_Address_Start[6];  //[Address High byte; mid byte; low byte; Data0; Data1] Array to be filled
uint32_t Sample_Length=0;
uint32_t Current_Sample_Count=0;
uint8_t bulkDataBuff[3] = {Write,0x00,0x00};

	void SetVcoreUp (unsigned int level)
	{
	  // Open PMM registers for write
	  PMMCTL0_H = PMMPW_H;
	  // Set SVS/SVM high side new level
	  SVSMHCTL = SVSHE + SVSHRVL0 * level + SVMHE + SVSMHRRL0 * level;
	  // Set SVM low side to new level
	  SVSMLCTL = SVSLE + SVMLE + SVSMLRRL0 * level;
	  // Wait till SVM is settled
	  while ((PMMIFG & SVSMLDLYIFG) == 0);
	  // Clear already set flags
	  PMMIFG &= ~(SVMLVLRIFG + SVMLIFG);
	  // Set VCore to new level
	  PMMCTL0_L = PMMCOREV0 * level;
	  // Wait till new level reached
	  if ((PMMIFG & SVMLIFG))
	    while ((PMMIFG & SVMLVLRIFG) == 0);
	  // Set SVS/SVM low side to new level
	  SVSMLCTL = SVSLE + SVSLRVL0 * level + SVMLE + SVSMLRRL0 * level;
	  // Lock PMM registers for write access
	  PMMCTL0_H = 0x00;
	}

	// Initializes the USCI_A1 module as a UART, using baudrate settings in
	// bcUart.h.  The baudrate is dependent on SMCLK speed.
	void bcUartInit(void)
	{
	    // Always use the step-by-step init procedure listed in the USCI chapter of
	    // the F5xx Family User's Guide
		  P4SEL |= BIT4+BIT5;         // Configure these pins as TXD/RXD

	    UCA1CTL1 |= UCSWRST;        // Put the USCI state machine in reset
	    UCA1CTL1 |= UCSSEL__SMCLK;  // Use SMCLK as the bit clock

	    //Set Baud rate for 2MhZ @ 9600 baud
		 UCA1BR0 = UCA1_BR0;
		 UCA1BR1 = UCA1_BR1;
		 UCA1MCTL = (UCA1_BRF << 0) | (UCA1_BRS << 3) ;
	    UCA1CTL1 &= ~UCSWRST;       // Take the USCI out of reset

	}

	void Spi_Init_UCB0(void)
	{
		//Configure for SPI module and external spi flash (UCB!)
		MISODIR;
		MOSIDIR;
		SPIClkDIR;
		MISOSEL;
		MOSISEL;
		SPIClkSEL;
		CSDIR;
		CS_High();//CS High
		__delay_cycles(500); // let SPI_Flash startup
		UCB0CTL1 = UCSWRST;
		UCB0CTL0 |= (UCMSB | UCMST | UCMODE_0 | UCCKPH |UCSYNC);
		UCB0CTL1 |= UCSSEL_2;
		UCB0CTL1 &= ~(UCSWRST);
	}


int main(void) {




    WDTCTL = WDTPW | WDTHOLD;	// Stop watchdog timer

	  //--- Configure  Clock ---
	SetVcoreUp (PMMCOREV_1);
	SetVcoreUp (PMMCOREV_2);
	SetVcoreUp (PMMCOREV_3);

	  UCSCTL3 = SELREF_2;                       // Set DCO FLL reference = REFO
	  UCSCTL4 |= SELA_2;                        // Set ACLK = REFO
	  UCSCTL0 = 0x0000;                         // Set lowest possible DCOx, MODx

	  // Loop until XT1,XT2 & DCO stabilizes - In this case only DCO has to stabilize
	  do
	  {
	    UCSCTL7 &= ~(XT2OFFG + XT1LFOFFG + DCOFFG);
	                                            // Clear XT2,XT1,DCO fault flags
	    SFRIFG1 &= ~OFIFG;                      // Clear fault flags
	  }while (SFRIFG1&OFIFG);                   // Test oscillator fault flag

	  __bis_SR_register(SCG0);                  // Disable the FLL control loop
	  UCSCTL1 = DCORSEL_5;                      // Select DCO range 16MHz operation
	  UCSCTL2 |= 249;                           // Set DCO Multiplier for 8MHz
	                                            // (N + 1) * FLLRef = Fdco
	                                            // (249 + 1) * 32768 = 8MHz
	  __bic_SR_register(SCG0);                  // Enable the FLL control loop
	  UCSCTL5 = DIVS__4;  //Divide SMCLK to 2MHz

	  // Worst-case settling time for the DCO when the DCO range bits have been
	  // changed is n x 32 x 32 x f_MCLK / f_FLL_reference. See UCS chapter in 5xx
	  // UG for optimization.
	  // 32 x 32 x 8 MHz / 32,768 Hz = 250000 = MCLK cycles for DCO to settle
	  __delay_cycles(250000);

	  //LED SETup for Debug
	  P1DIR |= (BIT6 + BIT0);
	  P4DIR |= BIT7;

    bcUartInit();

	Spi_Init_UCB0();
	
	_EINT();                      // enable interrupts
	Stage =0; //Set Stage
	UCA1IFG =0; //clear interrupt flags
	UCA1IE = UCRXIE;           // Enable the RX interrupt.  Now, when bytes are
								// rcv'ed, the USCI_A1 vector will be generated
	LEDRecOn();
	LED1On();
	LPM0;

	///Set Up Spi Flash
	//Change Status Register
	CS_Low();
	SpiTxBuff = Write_Enable;
	while ((BusyCheck));
	CS_High();
	__delay_cycles(cycle_delay);

	CS_Low();
	SpiTxBuff = Write_Stat;
	while ((BusyCheck));
	SpiTxBuff = Status;
	while ((BusyCheck));
	CS_High();
	__delay_cycles(cycle_delay);

	//Chip Erase
	CS_Low();
	SpiTxBuff = Write_Enable;
	while ((BusyCheck));
	CS_High();
	__delay_cycles(cycle_delay);


	if(chipErase)
	{
		CS_Low();
		SpiTxBuff = Chip_Erase;
		while ((BusyCheck));
		CS_High();
		__delay_cycles(cycle_delay);

		//Poll Status Register until erase finished
		CS_Low();

		SpiTxBuff =Read_Stat;
		while ((BusyCheck));
		SpiTxBuff = dummy;
		SpiRxFlagClear();
		SpiRxIntEnable();
		_BIS_SR(LPM0_bits); //go to LPM to wait for BUSY check
		while ((BusyCheck));
		CS_High();
		SpiRxIntDisable();
		__delay_cycles(cycle_delay);
	}
	//disbale writes to Spi flash

	CS_Low();
	SpiTxBuff = Write_Disable;
	while ((BusyCheck));
	CS_High();
	__delay_cycles(cycle_delay);

	//Spi Flash Is now setup for writes. Must Send WREN again before writing.
	LEDRecOn();
	//Spiflash is now ready for continuous write.

	while(1){

		//Lets get data from BC Uart for Length and Address
	  //LED Flash to say ready for GUI?

		if(bcBuffCount == 7) //If 7, just returned from a write. Need to Disable Writes.
		{
			//disbale writes to Spi flash
			CS_Low();
			SpiTxBuff = Write_Disable;
			while ((BusyCheck));
			CS_High();
			__delay_cycles(cycle_delay);
		}

		bcBuffCount=0;
		Current_Sample_Count=0;
		Sample_Length =0;



		UCA1IFG =0; //clear interrupt flags
		UCA1IE = UCRXIE;           // Enable the RX interrupt.  Now, when bytes are
									// rcv'ed, the USCI_A1 vector will be generated
		LEDRecOff();
		//wait in LPM
		if(Stage ==0) //Special case-- needed for erase checks
		{
			LPM0;
		}
		LPM0;
		LED2Tog();
	 //If we get here, then length  and start address + 2 data bytes are in buffers.

	    //Setup Start Address
		LEDRecOn();
		CS_Low();
		SpiTxBuff = Write_Enable;
		while ((BusyCheck));
		CS_High();
		__delay_cycles(cycle_delay);

		CS_Low();
		SpiTxBuff =Write;
		while((BusyCheck));
		SpiTxBuff = Sample_Address_Start[0];//Address high
		while((BusyCheck));
		SpiTxBuff = Sample_Address_Start[1];//Address mid
		while((BusyCheck));
		SpiTxBuff = Sample_Address_Start[2];//Address low
		while((BusyCheck));
		SpiTxBuff = Sample_Address_Start[3];//Data0
		while((BusyCheck));
		SpiTxBuff = Sample_Address_Start[4];//Data1
		while((BusyCheck));
		CS_High();
		__delay_cycles(cycle_delay);

		bcBuffCount =0; //reset buffer counter
		Stage =3;

		UCA1IFG =0; //clear interrupt flags
		UCA1IE = UCRXIE;           // Enable the RX interrupt.  Now, when bytes are
									// rcv'ed, the USCI_A1 vector will be generated


		//Go to LPM0 for Bulk Data Transfer
		LPM0;



	}

}
//Spi RX ISR -- Used for Read Status command on Spi Flash to poll Status register
#pragma vector=USCI_B0_VECTOR
__interrupt void USCI_B0_ISR(void)
{	uint8_t temp =0;

  switch(__even_in_range(UCB0IV,4))
  {
    case 0: break;                          // Vector 0 - no interrupt
    case 2:                                 // Vector 2 - RXIFG
    	temp = SpiRxBuff;
    	 while ((BusyCheck));                // USCI_B0 TX buffer ready?
    	if((temp & 0x01))
    	{
    		//still waiting on page write
    		    		 SpiTxBuff = dummy;
    	}
    	else{
    		// ready for new page
    		//exit out of LPmode
    		    		LPM0_EXIT;
    	}
    	break;
    case 4:

    	break;

  }
}

// The USCI_A1 receive interrupt service routine (ISR).  Executes every time a
// byte is received on the back-channel UART.
#pragma vector=USCI_A1_VECTOR
__interrupt void bcUartISR(void)
{ uint32_t temp=0;

	if(Stage ==0) //Erase Check
	{
		temp = bcUART_RX;
		if(temp == 0xAA)  //Will Erase Chip
		{
			chipErase =1;
			Stage =1;
			LPM0_EXIT;
		}
		else if (temp ==0x55)//Will Not Erase Chip
		{
			chipErase =0;
			Stage =1;
			LPM0_EXIT;
		}
		else
		{
			Stage =0; //Anything else ignore
		}
	}
	else if(Stage ==1) // Get Sample Length
	{

		if(bcBuffCount ==1)
		{	//high byte
			temp = bcUART_RX;
			Sample_Length |= (temp & 0x0000FF)<<16;
			bcBuffCount =2;
		}
		else if(bcBuffCount ==2)
		{	//midbyte
			temp = bcUART_RX;
			Sample_Length |= (temp & 0x0000FF)<<8;
			bcBuffCount =3 ; //reset
		}
		else if(bcBuffCount ==3)
		{	//midbyte
			temp = bcUART_RX;
			Sample_Length |= bcUART_RX;
			bcBuffCount =0 ; //reset
			Stage = 2; //Next Stage
		}
		else//bcBuffCount =0
		{ //low byte
			temp = bcUART_RX;
			Sample_Length |= (temp & 0x0000FF)<<24;

			bcBuffCount =1;
		}

	}
	else if(Stage ==2)//Get Sample Destination Address
	{
		LED2Tog();
		Sample_Address_Start[bcBuffCount]= bcUART_RX;
		bcBuffCount++;
		if(bcBuffCount>=5) //Got full Destination address stored from high byte @[0] and low byte @[2] then first two data bytes[3,4]
		{
			Stage =3; //Next bytes are data bulk
			bcBuffCount=0;//reset
			UCA1IE &= ~(UCRXIE); //disable bcUART interrupts
			Current_Sample_Count =2;
			LPM0_EXIT;
		}
	}
	else if(Stage ==3)//Bulk Data Transfer///
	{
		if(bcBuffCount) //even count -- second byte
		{
			bulkDataBuff[1] = bcUART_RX;  //store second byte and send 2
				CS_Low();
				SpiTxBuff =Write;
				while((BusyCheck));
				SpiTxBuff = bulkDataBuff[0];
				while((BusyCheck));
				SpiTxBuff = bulkDataBuff[1];
				while((BusyCheck));
				__delay_cycles(cycle_delay);
				CS_High();
				if(bcBuffCount ==7)
				{
					Stage = 0;
					UCA1IE &= ~(UCRXIE); //disable bcUART interrupts
					//bcBuffCount =0;
					LPM0_EXIT;

				}
				else
				{
					bcBuffCount =0; //reset count
				}
		}
		else //First Byte
		{
			bcBuffCount++; // Increase count to know you will be on second byte
			bulkDataBuff[0] = bcUART_RX;  //Store First Byte
			Current_Sample_Count += 2;
			if(Current_Sample_Count >= Sample_Length) ///check for full sample loaded
			{
				if(Sample_Length & 0x000001) //address odd length
				{
					CS_Low();
					SpiTxBuff =Write;
					while((BusyCheck));
					SpiTxBuff = bulkDataBuff[0];
					while((BusyCheck));
					SpiTxBuff = 0x00;
					while((BusyCheck));
					__delay_cycles(cycle_delay);
					CS_High();
					Stage = 0;
					Current_Sample_Count=0;
					bcBuffCount =7;
					UCA1IE &= ~(UCRXIE); //disable bcUART interrupts
					LPM0_EXIT;
				}
				else
				{
					bcBuffCount =7;
				}

			}
		}
	}
	else
	{
			//code should not get here
	}

}

