/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
/*
 * Playback_HAL.h
 *
 *  Created on: Jun 5, 2014 By: JH
 *  Edit:	JH	02/16	Conversion to OneMCU Standards
 *  					Move User Define area to Playback_UserDefine.h
 *                      PROCESSORS #define removed. CCS will know determine MCU
 *
 */

/*****************************************************************************/
/*   Playback Fucntioanlity for PWM DAC output of sound files 
/ *  with use of external SPI. To be used with Playback_main.c              */
/*****************************************************************************/

#ifndef PROCESSOR_H_
#define PROCESSOR_H_

#include <msp430.h>
#include <stdint.h>
#include <stdbool.h>
#include "Playback_UserDefine.h"

//*** Playback Frequency Options for FREQUENCY_SELECT
//***
#define FREQUENCY_8kHz   0x01
#define FREQUENCY_16kHz  0x02
#define FREQUENCY_22kHz  0x04  //only available on 5529
#define FREQUENCY_44kHz  0x08  //only available on 5529



//*** Resolution definition
// 8-bit resolution supported only. Future devices with faster 
// device speed could support more options.
//***
#define Resolution 0x0FF 

//*** The following #defines are hardware configurations for the different
// Launchpads. These defines include pin configurations,
// and Timer Register settings
//***
#if defined(__MSP430F5529__)
//Audio Amp Control Defines
#if(AUDIOAMPSHUTDOWN_ALTERNATE)
#define GPIO_AudioAmp_Direction         (P1DIR |= BIT3) // Audio Amp Reset Setup
#define GPIO_AudioAmp_Disable           (P1OUT |= BIT3) // Turns Amp off
#define GPIO_AudioAmp_Enable            (P1OUT &= ~BIT3)// Turns AMP on
#else
#define GPIO_AudioAmp_Direction         (P2DIR |= BIT6) // Audio Amp Reset Setup
#define GPIO_AudioAmp_Disable           (P2OUT |= BIT6) // Turns Amp off
#define GPIO_AudioAmp_Enable            (P2OUT &= ~BIT6)// Turns AMP on
#endif
#if (DEBUG)
//Debug Defines
#define GPIO_DebugPin1_Toggle           (P4OUT ^= BIT1)
#define GPIO_DebugPin0_Toggle           (P4OUT ^= BIT2)
#define GPIO_Debug_InitDIR              (P4DIR |= BIT1 | BIT2)
#define GPIO_Debug_InitOut              (P4OUT &= ~(BIT2 | BIT1))
#define GPIO_Debug_SMCLKOutput_InitSEL  (P2SEL |= BIT2)
#define GPIO_Debug_SMCLKOutput_InitDIR  (P2DIR |= BIT2)
#endif
//Button Defines 
#define GPIO_Button1_REN                (P2REN |= BIT1)
#define GPIO_Button2_REN                (P1REN |= BIT1)
#define GPIO_Button1_INPUT              (P2DIR &= ~(BIT1))
#define GPIO_Button2_INPUT              (P1DIR &= ~(BIT1))
#define GPIO_Button1_OUT                (P2OUT |= BIT1)
#define GPIO_Button2_OUT                (P1OUT |= BIT1)
#define GPIO_Button1_IES        (P2IES |= BIT1)     // interrupt on falling edge
#define GPIO_Button2_IES        (P1IES |= BIT1)     // interrupt on falling edge
#define GPIO_Button1_IFGClear   (P2IFG &= ~(BIT1))  // clear interrupt flags
#define GPIO_Button2_IFGClear   (P1IFG &= ~(BIT1))  // clear interrupt flags
#define GPIO_Button1_Disable            (P2IE &= ~(BIT1))
#define GPIO_Button2_Disable            (P1IE &= ~(BIT1))
#define GPIO_Button1_Enable             (P2IE |= BIT1)
#define GPIO_Button2_Enable             (P1IE |= BIT1)
#define GPIO_Button1_IFClear            (P2IFG &= ~(BIT1))
#define GPIO_Button2_IFClear            (P1IFG &= ~(BIT1))
#define GPIO_Button1_Check              (P2IFG & BIT1)
#define GPIO_Button2_Check              (P1IFG & BIT1)
#define GPIO_Button1_ISRVector          (PORT2_VECTOR)
#define GPIO_Button2_ISRVector          (PORT1_VECTOR)
#define GPIO_Button1_ISR                (port2_ISR)
#define GPIO_Button2_ISR                (port1_ISR)
//Timer Defines 
#if(AUDIOPWM_ALTERNATE)
#define Timer_ISRVector                 (TIMER2_A0_VECTOR)
#define Timer_ISR                       (TIMER2_A0_ISR)
#define GPIO_TimerPWM_DIR               (P2DIR |= BIT4)   //output
#define GPIO_TimerPWM_SEL               (P2SEL |= BIT4)   //timer output
#define Timer_Mode_Stop                 (TA2CTL = MC0)
#define Timer_Mode_UP                   (TA2CTL |= MC_1)
#define Timer_PWM_CCR                   (TA2CCR1)
#define Timer_IV                        (TA2IV)
#define Timer_PWM_CCRCase               (0x0E)
#else
#define Timer_ISRVector                 (TIMER1_A0_VECTOR)
#define Timer_ISR                       (TIMER1_A0_ISR)
#define GPIO_TimerPWM_DIR               (P2DIR |= BIT0)   //output
#define GPIO_TimerPWM_SEL               (P2SEL |= BIT0)   //timer output
#define Timer_Mode_Stop                 (TA1CTL = MC0)
#define Timer_Mode_UP                   (TA1CTL |= MC_1)
#define Timer_PWM_CCR                   (TA1CCR1)
#define Timer_IV                        (TA1IV)
#define Timer_PWM_CCRCase               (0x0E)
#endif
//Spi Defines
#define SPI_BusyCheck                   (UCB0STAT&UCBUSY)
#define SPI_TXBUF                       (UCB0TXBUF)
#define SPI_RXBUF                       (UCB0RXBUF)
#define GPIO_MISO_DIR                   (P3DIR &= ~(BIT1))
#define GPIO_MISO_SEL                   (P3SEL |= BIT1)
#define GPIO_MOSI_DIR                   (P3DIR |= BIT0)
#define GPIO_MOSI_SEL                   (P3SEL |= BIT0)
#define GPIO_SPICLK_DIR                 (P3DIR |= BIT2)
#define GPIO_SPICLK_SEL                 (P3SEL |= BIT2)
#if(CHIPSELECT_ALTERNATE)
#define GPIO_CS_DIR                     (P2DIR |= BIT2)
#define GPIO_CS_OUT_High                (P2OUT |= BIT2)
#define GPIO_CS_OUT_Low                 (P2OUT &= ~BIT2)
#else
#define GPIO_CS_DIR                     (P2DIR |= BIT3)
#define GPIO_CS_OUT_High                (P2OUT |= BIT3)
#define GPIO_CS_OUT_Low                 (P2OUT &= ~BIT3)
#endif
extern void setVcoreUp (unsigned int level)
    {
      // Open PMM registers for write
      PMMCTL0_H = PMMPW_H;
      // Set SVS/SVM high side new level
      SVSMHCTL = SVSHE + SVSHRVL0 * level + SVMHE + SVSMHRRL0 * level;
      // Set SVM low side to new level
      SVSMLCTL = SVSLE + SVMLE + SVSMLRRL0 * level;
      // Wait till SVM is settled
      while ((PMMIFG & SVSMLDLYIFG) == 0);
      // Clear already set flags
      PMMIFG &= ~(SVMLVLRIFG + SVMLIFG);
      // Set VCore to new level
      PMMCTL0_L = PMMCOREV0 * level;
      // Wait till new level reached
      if ((PMMIFG & SVMLIFG))
        while ((PMMIFG & SVMLVLRIFG) == 0);
      // Set SVS/SVM low side to new level
      SVSMLCTL = SVSLE + SVSLRVL0 * level + SVMLE + SVSMLRRL0 * level;
      // Lock PMM registers for write access
      PMMCTL0_H = 0x00;
    }
#elif defined(__MSP430FR5969__)
//Audio Amp Control Defines
#define GPIO_AudioAmp_Direction          (P1DIR |= BIT5) //Audio Amp Reset Setup
#define GPIO_AudioAmp_Disable            (P1OUT |= BIT5) // Turns Amp off
#define GPIO_AudioAmp_Enable             (P1OUT &= ~BIT5)// Turns AMP on
//Debug Defines
#if (DEBUG)
#define GPIO_DebugPin1_Toggle            (P3OUT ^= BIT5)
#define GPIO_DebugPin0_Toggle            (P3OUT ^= BIT6)
#define GPIO_Debug_InitDIR               (P3DIR |= BIT5 | BIT6)
#define GPIO_Debug_InitOut               (P3OUT &= ~(BIT5 | BIT6))
#define GPIO_Debug_SMCLKOutput_InitDIR   (P3DIR |= BIT4)
#define GPIO_Debug_SMCLKOutput_InitSEL   (P3SEL1 |= BIT4)
#endif
//Button Defines 
#define GPIO_Button1_REN                 (P4REN |= BIT5)
#define GPIO_Button2_REN                 (P1REN |= BIT1)
#define GPIO_Button1_OUT                 (P4OUT |= BIT5)
#define GPIO_Button2_OUT                 (P1OUT |= BIT1)
#define GPIO_Button1_INPUT               (P4DIR &= ~(BIT5))
#define GPIO_Button2_INPUT               (P1DIR &= ~(BIT1))
#define GPIO_Button1_IES        (P4IES |= BIT5)     // interrupt on falling edge
#define GPIO_Button2_IES        (P1IES |= BIT1)     // interrupt on falling edge
#define GPIO_Button1_IFGClear   (P4IFG &= ~(BIT5))  // clear interrupt flags)
#define GPIO_Button2_IFGClear   (P1IFG &= ~(BIT1))  // clear interrupt flags)
#define GPIO_Button1_Disable             (P4IE &= ~(BIT5))
#define GPIO_Button2_Disable             (P1IE &= ~(BIT1))
#define GPIO_Button1_Enable              (P4IE |= BIT5)
#define GPIO_Button2_Enable              (P1IE |= BIT1)
#define GPIO_Button1_IFClear             (P4IFG &= ~(BIT5))
#define GPIO_Button2_IFClear             (P1IFG &= ~(BIT1))
#define GPIO_Button1_Check               (P4IFG & BIT5)
#define GPIO_Button2_Check               (P1IFG & BIT1)
#define GPIO_Button1_ISRVector           (PORT4_VECTOR)
#define GPIO_Button2_ISRVector           (PORT1_VECTOR)
#define GPIO_Button1_ISR                 (port4_ISR)
#define GPIO_Button2_ISR                 (port1_ISR)
//Timer Defines
#define Timer_ISRVector                  (TIMER1_A0_VECTOR)
#define Timer_ISR                        (TIMER1_A0_ISR)
#define GPIO_TimerPWM_DIR                (P1DIR |= BIT2)  
#define GPIO_TimerPWM_SEL                (P1SEL0 |= BIT2); \
                                         (P1SEL1 &= ~(BIT2))
#define Timer_Mode_Stop                  (TA1CTL = MC0)
#define Timer_Mode_UP                    (TA1CTL |= MC_1)
#define Timer_PWM_CCR                    (TA1CCR1)
#define Timer_IV                         (TA1IV)
#define Timer_PWM_CCRCase                (0x0E)
//Spi Defines
#define SPI_BusyCheck                    (UCB0STAT&UCBUSY)
#define SPI_TXBUF                        (UCB0TXBUF)
#define SPI_RXBUF                        (UCB0RXBUF)
#define GPIO_MISO_DIR                    (P1DIR &= ~(BIT7))
#define GPIO_MISO_SEL                    (P1SEL1 |= BIT7);
#define GPIO_MOSI_DIR                    (P1DIR |= BIT6)
#define GPIO_MOSI_SEL                    (P1SEL1 |= BIT6);
#define GPIO_SPICLK_DIR                  (P2DIR |= BIT2)
#define GPIO_SPICLK_SEL                  (P2SEL1 |= BIT2);
#if(CHIPSELECT_ALTERNATE)
#define GPIO_CS_DIR                      (P3DIR |= BIT0)
#define GPIO_CS_OUT_High                 (P3OUT |= BIT0)
#define GPIO_CS_OUT_Low                  (P3OUT &= ~BIT0)
#else
#define GPIO_CS_DIR                      (P1DIR |= BIT4)
#define GPIO_CS_OUT_High                 (P1OUT |= BIT4)
#define GPIO_CS_OUT_Low                  (P1OUT &= ~BIT4)
#endif

#elif defined(__MSP430G2553__)
//Audio Amp Control Defines
#define GPIO_AudioAmp_Direction     (P2DIR |= BIT5)     // Audio Amp Reset Setup
#define GPIO_AudioAmp_Disable            (P2OUT |= BIT5)     // Turns Amp off
#define GPIO_AudioAmp_Enable             (P2OUT &= ~BIT5)    // Turns AMP on
//Debug Defines
#if (DEBUG)
#define GPIO_DebugPin1_Toggle            (P2OUT ^= BIT1)
#define GPIO_DebugPin0_Toggle            (P2OUT ^= BIT2)
#define GPIO_Debug_InitDIR               (P2DIR |= BIT1 | BIT2)
#define GPIO_Debug_InitOut               (P2OUT &= ~(BIT2 | BIT1))
#define GPIO_Debug_SMCLKOutput_InitDIR   (P1DIR |= BIT4)
#define GPIO_Debug_SMCLKOutput_InitSEL   (P1SEL |= BIT4)
#endif
//Button Defines
#define GPIO_Button1_OUT                 (P1OUT |= (BIT3))
#define GPIO_Button1_INPUT               (P1DIR &= ~(BIT3))
#define GPIO_Button1_REN                 (P1REN |= (BIT3))   // enable pullup
#define GPIO_Button1_IES         (P1IES |= (BIT3))   // interrupt on rising edge
#define GPIO_Button1_IFGClear    (P1IFG &= ~(BIT3))  // clear interrupt flags)
#define GPIO_Button1_Disable             (P1IE &= ~(BIT3))
#define GPIO_Button1_Enable              (P1IE |= (BIT3))
#define GPIO_Button1_IFClear             (P1IFG &= ~(BIT3))
#define GPIO_Button1_Check               (P1IFG & BIT3)
#define GPIO_Button1_ISRVector           (PORT1_VECTOR)
#define GPIO_Button1_ISR                 (port1_ISR)
//Timer Defines
#define Timer_ISRVector                  (TIMER0_A0_VECTOR)
#define Timer_ISR                        (timerA0_interrupt)
#define GPIO_TimerPWM_DIR                (P2DIR |= BIT6)    //output
#define GPIO_TimerPWM_SEL                (P2SEL = BIT6)     //timer output
#define Timer_Mode_Stop                  (TA0CTL = MC0)
#define Timer_Mode_UP                    (TA0CTL |= MC_1)
#define Timer_PWM_CCR                    (TA0CCR1)
#define Timer_IV                         (TA0IV)
#define Timer_PWM_CCRCase                (0x0A)
//Spi Defines
#define SPI_BusyCheck                    (UCB0STAT&UCBUSY)
#define SPI_TXBUF                        (UCB0TXBUF)
#define SPI_RXBUF                        (UCB0RXBUF)
#define GPIO_MISO_DIR                    (P1DIR &= ~(BIT6))
#define GPIO_MISO_SEL                    (P1SEL |= BIT6); \
                                         (P1SEL2 |= BIT6)
#define GPIO_MOSI_DIR                    (P1DIR |= BIT7)
#define GPIO_MOSI_SEL                    (P1SEL |= BIT7); \
                                         (P1SEL2 |= BIT7)
#define GPIO_SPICLK_DIR                  (P1DIR |= BIT5)
#define GPIO_SPICLK_SEL                  (P1SEL |= BIT5); \
                                         (P1SEL2 |= BIT5)
#if(CHIPSELECT_ALTERNATE)
#define GPIO_CS_DIR                      (P2DIR |= BIT7)
#define GPIO_CS_OUT_High                 (P2OUT |= BIT7)
#define GPIO_CS_OUT_Low                  (P2OUT &= ~BIT7)
#else
#define GPIO_CS_DIR                      (P2DIR |= BIT4)
#define GPIO_CS_OUT_High                 (P2OUT |= BIT4)
#define GPIO_CS_OUT_Low                  (P2OUT &= ~BIT4)
#endif
#endif

void setupSamplingFrequency(void)
{
    // Set up timer for PWM playback

        #if defined(__MSP430F5529__)
          _DINT();
            #if(AUDIOPWM_ALTERNATE)
              TA2R =0;
              TA2CCR0 = Resolution;                   // PWM Resolution = 8bits
              TA2CCR2 = 0;
              TA2CCTL0 = 0;
              TA2CCTL2 = OUTMOD_7 ;
              TA2CTL = TACLR | TASSEL_2 | TAIE ;
              _EINT();
              TA2CCTL0 |= CCIE;
            #else
              TA1CCR0 = Resolution;                   // PWM Resolution = 8bits
              TA1CCR1 = 0;
              TA1CCTL1 = OUTMOD_7 ;
              TA1CTL = TACLR | TASSEL_2 | TAIE ;
              _EINT();
              TA1CCTL0 |= CCIE;
            #endif
        #elif defined(__MSP430FR5969__)
          _DINT();
          TA1R =0;
          TA1CCR0 = Resolution;                       // PWM Resolution = 8bits
          TA1CCR1 = 0;
          TA1CCTL0 = 0;
          TA1CCTL1 = OUTMOD_7;
          TA1CTL = TACLR | TASSEL_2 | TAIE ;
          _EINT();
          TA1CCTL0 |= CCIE;
        #elif defined(__MSP430G2553__)
          _DINT();
          TA0R =0;
          TA0CCR0 = Resolution;                       // PWM Resolution = 8bits
          TA0CCR1 = 0;
          TA0CCTL0 = 0;
          TA0CCTL1 = OUTMOD_7 ;
          TA0CTL = TACLR | TASSEL_2 | TAIE ;
          _EINT();
          TA0CCTL0 |= CCIE;
        #endif
}

unsigned char setupClockSystem(uint8_t Freq)
{   //Returns TRUE if error (invalid frequency input)
#if defined(__MSP430F5529__)
  //--- Configure  Clock ---
  setVcoreUp (PMMCOREV_1);
  setVcoreUp (PMMCOREV_2);
  setVcoreUp (PMMCOREV_3);

#if(DEBUG)
  //For Debug Testing
  P2DIR |= BIT2;                               // SMCLK set out to pins
  P2SEL |= BIT2;
#endif
  UCSCTL3 |= SELREF_2;                         // Set DCO FLL reference = REFO
  UCSCTL4 |= SELA_2;                           // Set ACLK = REFO
  __bis_SR_register(SCG0);                     // Disable the FLL control loop
  UCSCTL0 = 0x0000;                            // Set lowest possible DCOx, MODx
  if (Freq & FREQUENCY_22kHz)
   {
        UCSCTL1 = DCORSEL_6;             // Select DCO range ~22.58Mhz operation
        UCSCTL2 = FLLD_0 + 687;          // Set DCO Multiplier for ~22.58Mhz
                                         // (N + 1) * FLLRef = Fdco
                                         // (688 + 1) * 32768 = 22.58Mhz
                                         // Set FLL Div = fDCOCLK/2
        UCSCTL5 |= DIVS__4;              // divide SMCLK down to 5.6MHz for 22k 
                                         // sampling Freq with 8-bit resolution
   }
   else if (Freq & FREQUENCY_44kHz)
   {
        UCSCTL1 = DCORSEL_6;             // Select DCO range ~22.58Mhz operation
        UCSCTL2 = FLLD_0 + 688;          // Set DCO Multiplier for ~22.58Mhz
                                         // (N + 1) * FLLRef = Fdco
                                         // (688 + 1) * 32768 = 22.58Mhz
                                         // Set FLL Div = fDCOCLK/2, divide 
        UCSCTL5 |= DIVS__2;              // SMCLK down to ~11.29MHz for 44k 
                                         // sampling Freq with 8-bit resolution
   }
   else if(Freq & FREQUENCY_16kHz)
   {
        UCSCTL1 = DCORSEL_5;             // Select DCO range ~16.38Mhz operation
        UCSCTL2 = FLLD_0 + 499;          // Set DCO Multiplier for ~16.384Mhz
                                         // (N + 1) * FLLRef = Fdco
                                         // (499 + 1) * 32768 = 16.384Mhz
                                         // Set FLL Div = fDCOCLK/4, divide
        UCSCTL5 |= DIVS__4;              // SMCLK down to ~4.096MHz for 16k 
                                         // sampling Freq with 8-bit resolution
    }
    else if (Freq & FREQUENCY_8kHz)
    {
        UCSCTL1 = DCORSEL_5;             // Select DCO range ~16.38Mhz operation
        UCSCTL2 = FLLD_0 + 499;          // Set DCO Multiplier for ~16.384Mhz
                                         // (N + 1) * FLLRef = Fdco
                                         // (499 + 1) * 32768 = 16.384Mhz
                                         // Set FLL Div = fDCOCLK/8
         UCSCTL5 |= DIVS__8;             // divide SMCLK down to ~2.04MHz for 8k 
                                         // sampling Freq with 8-bit resolution
    }
    else return 1; //Error invalid frequency for MSP430F5529
  __bic_SR_register(SCG0);                      // Enable the FLL control loop

  // Worst-case settling time for the DCO when the DCO range bits have been
  // changed is n x 32 x 32 x f_MCLK / f_FLL_reference. See UCS chapter in 5xx
  // UG for optimization.
  // 32 x 32 x 16 MHz / 32,768 Hz = 500000 = MCLK cycles for DCO to settle
  __delay_cycles(500000);

  // Loop until XT1,XT2 & DCO stabilizes 
  // In this case loop until XT1 and DCO settle
  do
  {
    UCSCTL7 &= ~(XT2OFFG + XT1LFOFFG + DCOFFG); // Clear XT2,XT1,DCO fault flags
    SFRIFG1 &= ~OFIFG;                          // Clear fault flags
  }while (SFRIFG1&OFIFG);                       // Test oscillator fault flag

  return 0;
#elif defined(__MSP430FR5969__)
  if(Freq & ~(FREQUENCY_8kHz | FREQUENCY_16kHz))
  {
    return 2; //Error Invlaid Frequency for MSP430FR5969
  }
  else
  {
   // Clock System Setup --MCLK = 8MHz SMLK playback frequency based
   CSCTL0_H = CSKEY >> 8;                        // Unlock CS registers
   CSCTL1 = DCOFSEL_6;                           // Set DCO to 8MHz
   CSCTL2 = SELA__VLOCLK | SELS__DCOCLK | SELM__DCOCLK;// Set SMCLK = MCLK = DCO
                                                        // ACLK = VLOCLK
    if(Freq & FREQUENCY_16kHz)
    {
      CSCTL3 = DIVA__1 | DIVS__2 | DIVM__1;     // Set SMCLK to 4MHz
    }
    else if(Freq & FREQUENCY_8kHz)
    {
      CSCTL3 = DIVA__1 | DIVS__4 | DIVM__1;     // Set SMCLK to 2MHz
    }
    else return 1;                   // Playback Frequency not supported : Error

    CSCTL0_H = 0;                                 // Lock CS registers
    __delay_cycles(500000);
    return 0;
  }
#elif defined(__MSP430G2553__)

  if(Freq & ~(FREQUENCY_8kHz | FREQUENCY_16kHz))
  {
        return 3; //Error Invlaid Frequency for MSP430G2553
  }
  else
  {
    if (CALBC1_16MHZ==0xFF)              // If calibration constant erased
    {
        return 4;                        // do not load, return
                                         //Invalid calibration constant
    }
    DCOCTL = 0;                          // Select lowest DCOx and MODx settings
    BCSCTL1 = CALBC1_16MHZ;              // Set range
    DCOCTL = CALDCO_16MHZ;               // Set DCO step + modulation
    if(Freq & FREQUENCY_16kHz)
    {
        BCSCTL2 |= DIVS_2;                      // Divide SMCLK to 4MHz
    }
    else if(Freq & FREQUENCY_8kHz)
    {
        BCSCTL2 |= DIVS_3;                      // Divide SMCLK to 2 MHz
    }
    else return 3; //Error Invlaid Frequency for MSP430G2553
    __delay_cycles(500000);
    return 0;
    }
 #endif
}

void setupButtons(void)
{
  //--- Configure the two push buttons to cause interrupts ---
 #if defined (__MSP430G2553__)
  GPIO_Button1_REN;
  GPIO_Button1_INPUT;
  GPIO_Button1_IES;
  GPIO_Button1_IFGClear;
  GPIO_Button1_Enable;                        // activate switch interrupts
 #elif defined(__MSP430F5529__) || defined(__MSP430FR5969__)
  GPIO_Button1_INPUT;
  GPIO_Button1_OUT;
  GPIO_Button1_REN;
  GPIO_Button1_IES;
  GPIO_Button1_IFGClear;
  GPIO_Button1_Enable;                        // activate switch interrupts
  GPIO_Button2_INPUT;
  GPIO_Button2_OUT;
  GPIO_Button2_REN;
  GPIO_Button2_IES;
  GPIO_Button2_IFGClear;
  GPIO_Button2_Enable;                        // activate switch interrupts
 #endif
}

void setupInitialSettings(void)
{
  // stop WDT
  WDTCTL = WDTPW + WDTHOLD;
 #if defined(__MSP430G2553__)
  P1DIR = 0xFF;
  P1OUT = 0x00;
  P2DIR = 0xFF;
  P2OUT = 0x00;
  P3DIR = 0xFF;
  P3OUT = 0x00;
 #elif defined(__MSP430FR5969__)
  P1DIR = 0xFF;
  P1OUT = 0x00;
  P2DIR = 0xFF;
  P2OUT = 0x00;
  P3DIR = 0xFF;
  P3OUT = 0x00;
  P4DIR = 0xFF;
  P4OUT = 0x00;
  PM5CTL0 &= ~LOCKLPM5; // Disable the GPIO power-on default high-impedance mode
                        // to activate previously configured port settings
 #elif defined(__MSP430F5529__)
  PADIR = 0xFF;
  PAOUT = 0x00;
  PBDIR = 0xFF;
  PBOUT = 0x00;
  PCDIR = 0xFF;
  PCOUT = 0x00;
  PDDIR = 0xFF;
  PDOUT = 0x00;
 #endif
}

void setupSPI(void)
{
#if (defined( __MSP430G2553__) || defined( __MSP430F5529__) || defined(__MSP430FR5969__ ))
  UCB0CTL1 = UCSWRST;
  UCB0CTL1 |= UCSSEL_2;
  //-- This Pre-processor directive only needed while a bug exists within 
  //-- the MSP430FR5969 header file. If SPI is no longer working for FR5969 
  //-- devices, the bug is likely fixed and this directive can be deleted.
  //-- In that case the #else statement below is valid for all processsors 
  //-- mentioned.
  #if defined(__MSP430FR5969__)
    UCB0CTL0 |= (UCMSB | UCMST | UCMODE_0 | UCCKPH |UCSYNC)>>8;
  #else
    UCB0CTL0 |= (UCMSB | UCMST | UCMODE_0 | UCCKPH |UCSYNC);
  #endif
  UCB0CTL1 &= ~(UCSWRST);
#endif
}

void disableButtons(void)
{
#if(defined (__MSP430G2553__))
  GPIO_Button1_Disable;
#elif(defined(__MSP430F5529__) || defined(__MSP430FR5969__))
  GPIO_Button1_Disable;
  GPIO_Button2_Disable;
#endif
}

void enableButtons(void)
{
#if(defined (__MSP430G2553__))
  GPIO_Button1_IFClear;                      // clear interrupt flags
  GPIO_Button1_Enable;                       // Re-enable buttons after play
#elif defined(__MSP430F5529__) || defined(__MSP430FR5969__)
  GPIO_Button1_IFClear;                      // clear interrupt flags
  GPIO_Button1_Enable;                       // Re-enable buttons after play
  GPIO_Button2_IFClear;                      // clear interrupt flags
  GPIO_Button2_Enable;                       // Re-enable buttons after play
#endif
}
#endif /* PROCESSOR_H_ */
