/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
/*
 * Playback_HAL.c
 *
 *  Created on: Jun 5, 2014 By: JH
 *  Edit:	JH	02/16	Conversion to OneMCU Standard
 *
 */

/*****************************************************************************/
//*   Playback Fucntioanlity for PWM DAC output of sound files with use of
//    external SPI
// *   To be used with Playback._HAL.h              */
/*****************************************************************************/

#include <msp430.h>
#include <stdbool.h>
#include <stdint.h>
#include "Playback_HAL.h"

extern unsigned char setupClockSystem(uint8_t Freq);
extern void setupSamplingFrequency(void);
extern void setupButtons(void);
extern void setupSPI(void);
extern void disableButtons(void);
extern void enableButtons(void);
extern void setupInitialSettings(void);

//*****************************************************************************
//defines
//*****************************************************************************
//*** Grab count for sound samples
//***
#define SPI_Grab_Count  10

//*** Length of buffer for sound samples grabbed. Equal to SPI_Grab_Count
//***
#define BufferLength    SPI_Grab_Count

//***Spi-Flash commands
//***
#define SPIFlash_Command_DummyWrite     0xF1
#define SPIFlash_Command_WriteEnable    0x06
#define SPIFlash_Command_Write          0xAD
#define SPIFlash_Command_ReadStatus     0x05
#define SPIFlash_Command_WriteDisable   0x04
#define SPIFlash_Command_ChipErase      0x60
#define SPIFlash_Command_WriteStatus    0x01
#define SPIFlash_Command_Read           0x03

//Channel Defines
//Assigned SPIFlashStartAddress and SPIFlashLength values during button push ISR
uint32_t Channel_1_SPIFlashStartAddress = 0;
uint32_t Channel_1_SPIFlashLength = 0;
uint32_t Channel1_SPIFlashCounter = 0;
uint16_t Channel1_WriteCounter;                   //write counter
uint16_t Channel1_ReadCounter;                    //read counter
uint8_t  Channel1_Buffer[BufferLength];
uint8_t  Channel1_GrabCounter = SPI_Grab_Count;     //grab counter
uint32_t channel1Address = 0;               //Address Variable
bool     Channel1_BufferBoundFlag = false;

//Audio Playback defines
bool ButtonPressedFlag = false;
bool AudioEndFlag = false;
unsigned char SampleSelectionState = 1;

unsigned char ClockErrorTrap = 0;                    //For clock error return

void runPlayback(void)
{

    AudioEndFlag = false;
    setupSamplingFrequency();
    _EINT();

    //Initial grab
    Channel1_GrabCounter = SPI_Grab_Count;
    channel1Address =
        (Channel1_SPIFlashCounter + Channel_1_SPIFlashStartAddress);
    GPIO_CS_OUT_Low;
    SPI_TXBUF = SPIFlash_Command_Read;
    while((SPI_BusyCheck));                      // TX buffer ready?

    #if (DEBUG)
    GPIO_DebugPin1_Toggle;
    #endif
    SPI_TXBUF = ((channel1Address & 0x00FF0000) >> 16);
    while((SPI_BusyCheck));                      // TX buffer ready?
    #if (DEBUG)
    GPIO_DebugPin1_Toggle;
    #endif
    SPI_TXBUF = ((channel1Address & 0x0000FF00) >> 8);
    while((SPI_BusyCheck));                      // TX buffer ready?
    #if (DEBUG)
    GPIO_DebugPin1_Toggle;
    #endif
    SPI_TXBUF = (channel1Address & 0x000000FF);
    while((SPI_BusyCheck));                      // TX buffer ready?
    #if (DEBUG)
    GPIO_DebugPin1_Toggle;
    #endif
    while(Channel1_GrabCounter)
    {
        SPI_TXBUF = SPIFlash_Command_DummyWrite;
        while((SPI_BusyCheck));                      // TX buffer ready?
        Channel1_Buffer[Channel1_WriteCounter] = SPI_RXBUF;
        Channel1_SPIFlashCounter++;
        Channel1_GrabCounter--;
        Channel1_WriteCounter++;
        #if (DEBUG)
        GPIO_DebugPin1_Toggle;
        #endif
    }
    GPIO_CS_OUT_High;
    Channel1_WriteCounter = 0;
    #if (DEBUG)
    GPIO_DebugPin1_Toggle;
    #endif
    Timer_Mode_UP;                            // start timer
    LPM0;
    //Spi grab loop until end of sample
    while(Channel1_SPIFlashCounter < Channel_1_SPIFlashLength)
    {
        //check for over-reach
        if((Channel1_SPIFlashCounter + SPI_Grab_Count) >
           Channel_1_SPIFlashLength)
        {
            Channel1_GrabCounter = Channel_1_SPIFlashLength -
                                   Channel1_SPIFlashCounter;
        }
        else
        {
            Channel1_GrabCounter = SPI_Grab_Count;
        }

        channel1Address = Channel1_SPIFlashCounter +
                          Channel_1_SPIFlashStartAddress;
        GPIO_CS_OUT_Low;
        SPI_TXBUF = SPIFlash_Command_Read;
        while((SPI_BusyCheck));                      // TX buffer ready?
        #if (DEBUG)
        GPIO_DebugPin1_Toggle;
        #endif
        SPI_TXBUF = ((channel1Address & 0x00FF0000) >> 16);
        while((SPI_BusyCheck));                      // TX buffer ready?
        #if (DEBUG)
        GPIO_DebugPin1_Toggle;
        #endif
        SPI_TXBUF = ((channel1Address & 0x0000FF00) >> 8);
        while((SPI_BusyCheck));                      // TX buffer ready?
        #if (DEBUG)
        GPIO_DebugPin1_Toggle;
        #endif
        SPI_TXBUF = (channel1Address & 0x000000FF);
        while((SPI_BusyCheck));                      // TX buffer ready?
        #if (DEBUG)
        GPIO_DebugPin1_Toggle;
        #endif
        while(Channel1_GrabCounter)
        {
            SPI_TXBUF = SPIFlash_Command_DummyWrite;
            while((SPI_BusyCheck));                      // TX buffer ready?
            Channel1_Buffer[Channel1_WriteCounter] = SPI_RXBUF;
            Channel1_SPIFlashCounter++;
            Channel1_GrabCounter--;
            Channel1_WriteCounter++;
            #if (DEBUG)
            GPIO_DebugPin1_Toggle;
            #endif
        }
        GPIO_CS_OUT_High;
        Channel1_WriteCounter = 0;
        #if (DEBUG)
        GPIO_DebugPin1_Toggle;
        #endif
        LPM0;
    }
    AudioEndFlag = true;
    Timer_Mode_Stop;
}

void main(void)
{
    setupInitialSettings();

    //--- Configure  Clock ---
    ClockErrorTrap = setupClockSystem(FREQUENCY_SELECT);
    if(ClockErrorTrap)
    {
        //error detected
        ClockErrorTrap = false;
        WDTCTL = WDTPW + ~(WDTHOLD);
        LPM3;
    }

    #if (DEBUG) //configure DEBUG pins
    ///debug pins
    GPIO_Debug_InitDIR;
    GPIO_Debug_InitOut;
    #endif

    setupButtons();

    //configure TimerPWM Output
    GPIO_TimerPWM_DIR;
    GPIO_TimerPWM_SEL;

    //Audio Amp Setup
    GPIO_AudioAmp_Direction;
    GPIO_AudioAmp_Enable;

    //Configure for SPI module
    GPIO_MISO_DIR;
    GPIO_MOSI_DIR;
    GPIO_SPICLK_DIR;
    GPIO_MISO_SEL;
    GPIO_MOSI_SEL;
    GPIO_SPICLK_SEL;
    GPIO_CS_DIR;
    GPIO_CS_OUT_High;                          //CS High
    __delay_cycles(500);                       // let SPI_Flash startup
    setupSPI();
    #if (DEBUG)
    GPIO_Debug_SMCLKOutput_InitDIR;            //Set SMCLK to output for debug
    GPIO_Debug_SMCLKOutput_InitSEL;
    #endif

    while(1)
    {
        _EINT();                                   // enable interrupts
        GPIO_AudioAmp_Disable;                 //turn off amp to save power
        //--- wait for key-press event, hold CPU in low-power mode ---
        _BIS_SR(LPM3_bits);                    // enter LPM0
        //*** process key-press event ***
        if(ButtonPressedFlag)                  //  pressed?
        {
            disableButtons();             // Disable buttons during play
            GPIO_AudioAmp_Enable;              //Turn on Amp
            runPlayback();               //Playback
            ButtonPressedFlag = false;
            enableButtons();
        }
    }
}

#pragma vector=Timer_ISRVector
__interrupt void Timer_ISR(void)
{
   #if (DEBUG)
    GPIO_DebugPin0_Toggle;
   #endif
    switch(__even_in_range(Timer_IV, Timer_PWM_CCRCase))  
    {                                      // Efficient switch-implementation
    case  0x0000:
        break;
    case Timer_PWM_CCRCase:     // Compare match interrupt (TACCR4 - duty cycle)
              Timer_PWM_CCR = Channel1_Buffer[Channel1_ReadCounter];
              Channel1_ReadCounter++;
              //setup bound control.
              if(Channel1_ReadCounter >= (SPI_Grab_Count - 1))    //buffer bound
              {
                 if(Channel1_BufferBoundFlag)
                 {
                     Channel1_BufferBoundFlag = false;
                     Channel1_ReadCounter = 0;
                 }
                 else
                 {
                     Channel1_BufferBoundFlag = true;
                     __bic_SR_register_on_exit(LPM0_bits);
                 }
             }
             // Audio Bound check
             else if(AudioEndFlag)                  //check low audio bound
             {
                 Timer_Mode_Stop;                   //Timer Stop
                 _bic_SR_register_on_exit(LPM0_bits);
             }
             break;
    default:
        break;
    }
    #if (DEBUG)
    GPIO_DebugPin0_Toggle;
    #endif
}

//The following ISRs demonstrate different techniques for cycling through 
//samples or repeating the same samples.The important aspect is to assign
// the start address and length of samples to be played.

#if defined(__MSP430G2553__)
     #pragma vector=GPIO_Button1_ISRVector
     __interrupt void GPIO_Button1_ISR(void)
     {
     //The following if statement will cycle through 3 sound samples 
     //defined in Playback_HAL.h
         if(SampleSelectionState == 1)
         {
             Channel_1_SPIFlashStartAddress = soundDataStartAddress2;
             Channel_1_SPIFlashLength = soundDataLength2;
             SampleSelectionState = 2;
         }
         else if(SampleSelectionState == 2)
         {
             Channel_1_SPIFlashStartAddress = soundDataStartAddress2;
             Channel_1_SPIFlashLength = soundDataLength2;
             SampleSelectionState = 0;
         }
         else
         {
             Channel_1_SPIFlashStartAddress = soundDataStartAddress2;
             Channel_1_SPIFlashLength = soundDataLength2;
             SampleSelectionState = 2;
         }
         Channel1_ReadCounter = 0;
         Channel1_WriteCounter = 0;
         Channel1_SPIFlashCounter = 0;
         GPIO_Button1_IFClear;               // clear all button interrupt flags
         ButtonPressedFlag = true;

         _bic_SR_register_on_exit(LPM3_bits);       // exit LPM3 on reti
     }

#elif defined(__MSP430F5529__) || defined(__MSP430FR5969__)
     #pragma vector=GPIO_Button1_ISRVector
     __interrupt void GPIO_Button1_ISR(void)
     {
         //The following if statement will cycle through 3 sound samples 
         //defined in Playback_HAL.h
         if(SampleSelectionState == 2)
         {
             Channel_1_SPIFlashStartAddress = soundDataStartAddress2;
             Channel_1_SPIFlashLength = soundDataLength2;
         }
         else if(SampleSelectionState == 1)
         {
             Channel_1_SPIFlashStartAddress = soundDataStartAddress1;
             Channel_1_SPIFlashLength = soundDataLength1;
         }
         else
         {
             Channel_1_SPIFlashStartAddress = soundDataStartAddress3;
             Channel_1_SPIFlashLength = soundDataLength3;
             SampleSelectionState = 0;
         }

         SampleSelectionState++;
         Channel1_ReadCounter = 0;
         Channel1_WriteCounter = 0;
         Channel1_SPIFlashCounter = 0;
         GPIO_Button1_IFClear;           //clear all button interrupt flags
         ButtonPressedFlag = true;

         _bic_SR_register_on_exit(LPM3_bits);             //exit LPM3 on return
     }

     //--Second LaunchPad Button -- Toggles Between  first two samples

     #pragma vector=GPIO_Button2_ISRVector
     __interrupt void GPIO_Button2_ISR(void)
     {
         //--The following toggles through the two audio samples
         if(SampleSelectionState)
         {
             Channel_1_SPIFlashStartAddress = soundDataStartAddress1;
             Channel_1_SPIFlashLength = soundDataLength1;
         }
         else
         {
             Channel_1_SPIFlashStartAddress = soundDataStartAddress2;
             Channel_1_SPIFlashLength = soundDataLength2;
         }
         SampleSelectionState ^= 1;
         Channel1_ReadCounter = 0;
         Channel1_WriteCounter = 0;
         Channel1_SPIFlashCounter = 0;
         GPIO_Button2_IFClear;                //clear all button interrupt flags
         ButtonPressedFlag = true;
         _bic_SR_register_on_exit(LPM3_bits);             //exit LPM3 on return
     }

#endif
